<?php
/**
 * COD24 Sefareshi Shipping
 * 
 * Package: COD24 Shipping
 * Author: Rasool Vahdati
 * DateTime: 2022/12/04 00:10
 * Last Modified Time: 2023/11/26 01:13:11
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */ 

if (!defined('ABSPATH')) {
    exit;
}

/*
 * Check if WooCommerce is active
 */
if(in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
    
    function cod24_sefareshi_shipping_method()
    {
        if (!class_exists('COD24_Sefareshi_Shipping')) {
            class COD24_Sefareshi_Shipping extends WC_Shipping_Method
            {
                /**
                 * Constructor for your shipping class
                 *
                 * @access public
                 * @return void
                 */
                public function __construct( $instance_id = 0 )
                {
                    $this->id                 = 'cod24_sefareshi';
                    $this->instance_id        = absint( $instance_id );
                    $this->method_title       = __('Cod24 Sefareshi Shipping', 'cod24-shipping');
                    $this->method_description = __('Sefareshi Shipping Method for Woocommerce', 'cod24-shipping');
                    $this->current_currency   = get_woocommerce_currency();
                    $this->supports           = array(
                        'shipping-zones',
                        'instance-settings'
                    );

                    // Availability & Countries
                    $this->availability = 'including';
                    $this->countries    = array(
                        'IR' // IR
                    );

                    $this->instance_form_fields = array(
                        'enabled' => array(
                            'title' 		=> __('Enable', 'cod24-shipping'),
                            'type' 			=> 'checkbox',
                            'label' 		=> __('Enable this shipping.', 'cod24-shipping'),
                            'default' 		=> 'yes',
                        ),
                        'title' => array(
                            'title' 		=> __('Title', 'cod24-shipping'),
                            'type' 			=> 'text',
                            'description' 	=> __('Title to be display on site', 'cod24-shipping'),
                            'default'		=> __('Cod24 Sefareshi Shipping', 'cod24-shipping')
                        )
                    );
                    $this->enabled           = $this->get_option( 'enabled' );
                    $this->title             = $this->get_option( 'title' );
                }
                
                /**
                 * Init your settings
                 *
                 * @access public
                 * @return void
                 */
                function init()
                {
                    // Load the settings API
                    $this->init_form_fields();
                    $this->init_settings();
                    
                    // Save settings in admin if you have any defined
                    add_action('woocommerce_update_options_shipping_' . $this->id, array(
                        $this,
                        'process_admin_options'
                    ));
                }
                
                /**
                 * Define settings field for this shipping
                 * @return void 
                 */
                function init_form_fields()
                {
                    
                    $this->form_fields = array(
                        
                        'enabled' => array(
                            'title' => __('Enable', 'cod24-shipping'),
                            'type' => 'checkbox',
                            'description' => __('Enable this shipping.', 'cod24-shipping')
                        ),
                        
                        'title' => array(
                            'title' => __('Title', 'cod24-shipping'),
                            'type' => 'text',
                            'description' => __('Title to be display on site', 'cod24-shipping'),
                            'default' => __('Sefareshi Shipping', 'cod24-shipping')
                        )
                        
                    );
                }
                
                public function is_available( $package = [] )
                {
                    $available = $this->is_enabled();

                    if ( empty( $package ) ) {
                        $available = false;
                    }
                    
                    if ( $package['destination']['country'] != 'IR' ) {
                        $available = false;
                    }
                
                    if ( is_null( $package['destination']['state'] ) ) {
                        $available = false;
                    }
                
                    if ( is_null( $package['destination']['city'] ) ) {
                        $available = false;
                    }

                    $user_delivery_address = COD24_API::find_city( $package['destination']['city'] );
                    if(empty($user_delivery_address['state_id']) && empty($user_delivery_address['city_id']))
                        $available = false;
                
                    return apply_filters( 'woocommerce_shipping_' . $this->id . '_is_available', $available, $package, $this );
                }
                
                /**
                 * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
                 *
                 * @access public
                 * @param mixed $package
                 * @return void
                 */
                public function calculate_shipping($package = array())
                {
                    $weight = 0;
                    $cost   = 0;
                    $country = $package["destination"]["country"];
                    
                    foreach ($package['contents'] as $item_id => $values) {
                        $_product = $values['data'];
                        $weight   = $weight + $_product->get_weight() * $values['quantity'];
                    }
                    
                    $unit = get_option('woocommerce_weight_unit');

                    // Check if the weight unit is in grams
                    if ($unit === 'g') {
                        // The weight is already in grams, do not convert.
                        $total_weight = $weight;
                    } else {
                        $total_weight = $weight * 1000; // Convert to grams
                    }
                    
                    $cart_price = WC()->cart->get_cart_contents_total();
                    
                    if( $this->current_currency == 'IRR' ) {
                        $total_cart_price = ceil($cart_price * 10);
                    } else {
                        $total_cart_price = $cart_price;
                    }
                    
                    // Get Price From Web Service API
                    // check payment type
                    //$chosen_payment_method = WC()->session->get('chosen_payment_method');
                    if ($package['destination']['is_cod'])
                        $pay_type = 0;
                    else
                        $pay_type = 1;
                    
                    // Service Type
                    $send_type = 0; // sefareshi service type is 1
                    
                    // get city
                    $delivery_city_name = WC()->customer->get_shipping_city();
                    $user_delivery_address = COD24_API::find_city( $delivery_city_name );
                    
                    if(!is_null($user_delivery_address['state_id']) && !is_null($user_delivery_address['city_id']))
                    {
                        // Get COD24 Rate
                        $params   = array(
                            'weight' => $total_weight,
                            'price' => $total_cart_price,
                            'state' => $user_delivery_address['state_id'],
                            'city' => $user_delivery_address['city_id'],
                            'pay_type' => $pay_type,
                            'send_type' => $send_type
                        );

                        $cod_rate = COD24_API::get_rate($params);
                        if(!empty($response['code']) && $response['code']== 401)
                        {
                            COD24_API::get_token();
                            $cod_rate = COD24_API::get_rate($params);
                        }
                        
                        $countryZones = array(
                            'IR' => ($cod_rate > 0) ? $cod_rate : 0
                        );
                        
                        $cost = $countryZones['IR'];
                        
                        // convert current currency
                        if( $this->current_currency == 'IRR' ) {
                            $shipping_cost = ceil($cost * 10);
                        } else {
                            $shipping_cost = $cost;
                        }
                        
                        $rate = array(
                            'id' => $this->id . $this->instance_id,
                            'label' => $this->title,
                            'cost' => $shipping_cost,
                            'calc_tax' => 'per_order'
                        );
                        
                        $this->add_rate($rate);
                    }
                }
            }
        }
    }
    
    add_action('woocommerce_shipping_init', 'cod24_sefareshi_shipping_method');
    
    function add_cod24_sefareshi_shipping($methods)
    {
        $methods['cod24_sefareshi'] = 'COD24_Sefareshi_Shipping';
        return $methods;
    }
    
    add_filter('woocommerce_shipping_methods', 'add_cod24_sefareshi_shipping');
    
    function cod24_sefareshi_validate_order($posted)
    {
        
        $packages = WC()->shipping->get_packages();
        
        $chosen_methods = WC()->session->get('chosen_shipping_methods');
        
        if (is_array($chosen_methods) && in_array('cod24_sefareshi_shipping', $chosen_methods)) {
            
            foreach ($packages as $i => $package) {
                
                if ($chosen_methods[$i] != "cod24_sefareshi") {
                    continue;
                }
                
                $COD24_Sefareshi_Shipping = new COD24_Sefareshi_Shipping();
                $min_weight            = 30; // gram
                $max_weight            = 20000; // gram
                $weight                 = 0;
                
                foreach ($package['contents'] as $item_id => $values) {
                    $_product = $values['data'];
                    $weight   = $weight + $_product->get_weight() * $values['quantity'];
                }
                
                $weight = wc_get_weight($weight, 'g');

                if ($weight >= $min_weight && $weight <= $max_weight) {
                    
                    // nothing
                
                } else {
                    
                    $message = sprintf(__('Sorry, %d must be between %d and %d', 'cod24-shipping'), $weight, $min_weight, $max_weight);
                    
                    $messageType = "error";
                    
                    if (!wc_has_notice($message, $messageType)) {
                        
                        wc_add_notice($message, $messageType);
                    }
                }
            }
        }
    }
    
    add_action('woocommerce_review_order_before_cart_contents', 'cod24_sefareshi_validate_order', 10);
    add_action('woocommerce_after_checkout_validation', 'cod24_sefareshi_validate_order', 10);
}