<?php

if (!defined('ABSPATH')) {
    exit;
} // Exit if accessed directly

if (!class_exists('Asokala_Order_Delivery')) {
    class COD24_WC_Settings
    {
        
        public static function init()
        {
            add_filter('woocommerce_settings_tabs_array', __CLASS__ . '::add_settings_tab', 50);
            add_action('woocommerce_settings_tabs_cod24_settings', __CLASS__ . '::settings_tab');
            add_action('woocommerce_update_options_cod24_settings', __CLASS__ . '::update_settings');

            add_action('update_option', __CLASS__ . '::update_option_wc_token' , 10, 3 );
        }
        
        /**
         * Add a new settings tab to the WooCommerce settings tabs array.
         *
         * @param array $settings_tabs Array of WooCommerce setting tabs & their labels, excluding the Subscription tab.
         * @return array $settings_tabs Array of WooCommerce setting tabs & their labels, including the Subscription tab.
         */
        public static function add_settings_tab($settings_tabs)
        {
            $settings_tabs['cod24_settings'] = __('COD24 Settings', 'cod24-shipping');
            return $settings_tabs;
        }
        
        /**
         * Uses the WooCommerce admin fields API to output settings via the @see woocommerce_admin_fields() function.
         *
         * @uses woocommerce_admin_fields()
         * @uses self::get_settings()
         */
        public static function settings_tab()
        {
            woocommerce_admin_fields(self::get_settings());
        }
        
        /**
         * Uses the WooCommerce options API to save settings via the @see woocommerce_update_options() function.
         *
         * @uses woocommerce_update_options()
         * @uses self::get_settings()
         */
        public static function update_settings()
        {
            woocommerce_update_options(self::get_settings());
        }
        
        /**
         * Get all the settings for this plugin for @see woocommerce_admin_fields() function.
         *
         * @return array Array of settings for @see woocommerce_admin_fields() function.
         */
        public static function get_settings()
        {
            $settings = array(
                'section_title' => array(
                    'name' => __('COD24 Settings', 'cod24-shipping'),
                    'type' => 'title',
                    'desc' => '',
                    'id' => 'wc_cod24_settings_section_title'
                ),
                'cod24_username' => array(
                    'name' => __('Username', 'cod24-shipping'),
                    'type' => 'text',
                    'id' => 'wc_cod24_username_panel'
                ),
                'cod24_password' => array(
                    'name' => __('Password', 'cod24-shipping'),
                    'desc' => __('The web service password is different from your panel password. Contact support to receive a password.', 'cod24-shipping'),
                    'type' => 'text',
                    'id' => 'wc_cod24_password_panel',
                ),
                'cod24_api_secure_connection_enabled' => array(
                    'name'     => __( 'Enable Secure Connection', 'cod24-shipping' ),
                    'desc'     => __( 'if this option is enable, the api connection will be secured', 'cod24-shipping' ),
                    'type' => 'checkbox',
                    'default' => 'yes',
                    'id' => 'wc_cod24_api_secure_connection_enabled'
                ),
                'cod24_free_shipping_cod_enabled' => array(
                    'name'     => __( 'Enable', 'cod24-shipping' ),
                    'desc'     => __( 'Enable free shipping when the cod payment method is available', 'cod24-shipping' ),
                    'type' => 'checkbox',
                    'default' => 'yes',
                    'id' => 'wc_cod24_free_shipping_cod_enabled'
                ),
                'cod24_shcod_enabled' => array(
                    'name'     => __( 'Enable shipping at the destination', 'cod24-shipping' ),
                    'desc'     => __( 'Enable', 'cod24-shipping' ),
                    'type' => 'checkbox',
                    'default' => 'yes',
                    'id' => 'wc_cod24_shcod_enabled'
                ),
                'section_end' => array(
                    'type' => 'sectionend',
                    'id' => 'wc_cod24_settings_section_end'
                )
            );
            return apply_filters('wc_cod24_settings', $settings);
        }

        /**
         * Update cod24 token when updated username panel
         * 
         * @param old_value
         * @param new_value
         * @param option_name
         * @return void
         */
        public static function update_option_wc_token( $old_value, $new_value, $option_name )
        {
            if( $option_name == 'wc_cod24_username_panel')
            {
                COD24_API::get_token();
            }
        }

    }
    COD24_WC_Settings::init();
}