<?php
/**
 * COD24 API
 * This class is connecting to COD24 API.
 * 
 * Package: COD24 Shipping
 * Author: Rasool Vahdati
 * DateTime: 2022/12/04 00:10:20
 * Last Modified Time: 2023/05/13 00:24:34
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */ 

if (!defined('ABSPATH')) {
	exit;
} // Exit if accessed directly

if (!class_exists('COD24_SHCOD')) {
	class COD24_SHCOD
	{

		/**
		 * Construct the plugin object
		 */
		public function __construct()
		{
            add_action( 'woocommerce_after_checkout_billing_form' , array(  &$this , 'add_shcod_checkbox_checkout' ) );
			add_action( 'wp_footer' , array(  &$this , 'shcod_fee_script' ) );
			add_action( 'wp_ajax_shipping_on_delivery' , array(  &$this , 'get_ajax_shipping_on_delivery_fee' ) );
			add_action( 'wp_ajax_nopriv_shipping_on_delivery' , array(  &$this , 'get_ajax_shipping_on_delivery_fee' ) );
			add_action( 'woocommerce_cart_calculate_fees' , array(  &$this , 'add_remove_shipping_on_delivery_fee' ) );
			add_action( 'woocommerce_checkout_update_order_meta' , array(  &$this , 'save_shipping_on_delivery_field' ) );

			// woocommerce change available payment gateways
			add_filter( 'woocommerce_available_payment_gateways' , array(  &$this , 'filter_gateways' ), 1000 );
			
		} // END public function __construct()
		
		/**
		 * Add shipping on delivery checkbox on checkout
		 * 
		 * @param void
		 * @return void
		 */
		public function add_shcod_checkbox_checkout()
		{
			$shipping_cost_on_delivery = get_option('wc_cod24_shcod_enabled');

			if( $shipping_cost_on_delivery == 'yes' )
			{
				$shipping_on_delivery_val = '0';
				if(WC()->session->get('shcod_fee'))
					$shipping_on_delivery_val = '1';
			
				woocommerce_form_field( 'shipping_on_delivery', array(
					'type'  => 'checkbox',
					'label' => __('I pay the shipping cost at the destination', 'cod24-shipping'),
					'class' => array( 'form-row-wide' ),
				), $shipping_on_delivery_val );
			}
		}

		/**
		 * Shipping on delivery fee script
		 * 
		 * @param void
		 * @return void
		 */
		public function shcod_fee_script()
		{
			if( ( is_checkout() && ! is_wc_endpoint_url() ) ) :
			?>
			<script type="text/javascript">
			jQuery( function($){
				if (typeof woocommerce_params === 'undefined')
					return false;

				$('input[name=shipping_on_delivery]').click( function(){
					var fee = $(this).prop('checked') === true ? '1' : '';

					$.ajax({
						type: 'POST',
						url: woocommerce_params.ajax_url,
						data: {
							'action': 'shipping_on_delivery',
							'shcod_fee': fee,
						},
						success: function (result) {
							$('body').trigger('update_checkout');
							console.log(result);
						},
					});
				});

				$( 'body' ).on( 'change', 'input[name="payment_method"]', function() {
					if( jQuery('input[name="payment_method"]:checked').val() == 'cod' )
					{
						jQuery('input[name=shipping_on_delivery]').attr('checked', false );
						jQuery('#shipping_on_delivery_field').slideUp(1000);
					}
					else
					{
						jQuery('#shipping_on_delivery_field').slideDown(1000);
					}
				});
			});
			</script>
			<?php
			endif;
		}


		/**
		 * Ajax handler for shipping on delivery fee
		 * 
		 */
		public function get_ajax_shipping_on_delivery_fee() {
			if ( isset($_POST['shcod_fee']) ) {
				WC()->session->set('shcod_fee', ($_POST['shcod_fee'] ? '1' : '0') );
				echo WC()->session->get('shcod_fee');
			}
			die();
		}

		/**
		 * Add or remove shipping on delivery fee
		 * 
		 * @param cart
		 * @return void
		 */
		public function add_remove_shipping_on_delivery_fee( $cart )
		{
			// Only on checkout
			if ( ( is_admin() && ! defined( 'DOING_AJAX' ) ) || is_cart() )
				return;

			if( WC()->session->get('shcod_fee') ) {
				$available_gateways = WC()->payment_gateways->get_available_payment_gateways();
				$first_gateway = reset( $available_gateways );
				WC()->session->set( 'chosen_payment_method', $first_gateway->id );
				WC()->cart->calculate_shipping();
				$fee_amount = (float) esc_attr( WC()->cart->get_shipping_total() );
				$cart->add_fee( __('shipping cost at the destination', 'cod24-shipping'), -$fee_amount );
			}
		}

		/**
		 * Change available payment gateways
		 * 
		 * @param gateways
		 * @return gateways
		 */
		public function filter_gateways( $gateways )
		{
			global $woocommerce;        
			//Remove a specific payment option
			if(isset(WC()->session) && WC()->session->get('shcod_fee') != null && WC()->session->get('shcod_fee'))
				unset($gateways['cod']);
			return $gateways;
		}

		/**
		 * Save shipping on delivery field in order detai 
		 *
		 * @param order_id
		 * @return void
		 */  
		public function save_shipping_on_delivery_field( $order_id )
		{ 
			if ( isset( $_POST['shipping_on_delivery'] ) ) update_post_meta( $order_id, '_is_shipping_cost_on_delivery', 'yes' );
		}

	} // END class COD24_Shipping
} // END if(!class_exists('COD24_Shipping'))

if (class_exists('COD24_SHCOD')) {
	// instantiate the plugin class
	new COD24_SHCOD();
}