<?php
/**
 * COD24 Woo
 * This class includes all woocommerce functions
 * 
 * Package: COD24 Shipping
 * Author: Rasool Vahdati
 * DateTime: 2022/12/04 00:10
 * Last Modified Time: 2023/11/26 13:50:46
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */ 

if (!defined('ABSPATH')) {
	exit;
} // Exit if accessed directly

if (!class_exists('COD24_WOO')) {
	class COD24_WOO
	{

		/**
		 * Construct the plugin object
		 */
		public function __construct()
		{
			// initialize scripts
			add_action( 'wp_enqueue_scripts' , array(  &$this , 'load_scripts' ) );

			// change woocommerce fields
            add_filter( 'woocommerce_billing_fields' , array( &$this , 'billing_fields' ) , 10, 2 );
			add_filter( 'woocommerce_shipping_fields' , array( &$this , 'shipping_fields' ) , 10, 2 );
			add_filter( 'woocommerce_form_field_city' , array( &$this , 'form_field_city' ) , 10, 4 );
			add_filter( 'woocommerce_checkout_fields' , array( &$this , 'override_checkout_fields' ) , 999 );
			add_filter( 'woocommerce_default_address_fields' , array( &$this , 'override_default_locale_fields' ) , 999 );

			// woocommerce cart shipping packages
			add_filter( 'woocommerce_cart_shipping_packages' , array( &$this , 'cart_shipping_packages' ) , 20, 1 );

			// woocommerce package rates
			add_filter( 'woocommerce_package_rates' , array( &$this , 'hide_shipping_when_free_is_available' ) , 100 );

			// set custom total weight in woocommerce thankyou
			add_action( 'woocommerce_thankyou' , array( &$this , 'checkout_save_total_weight_meta' ) );

			// order metabox & ajax handler
			add_action( 'add_meta_boxes' , array( &$this , 'order_detail_metabox' ) );
			add_action( 'save_post_shop_order', array( &$this , 'save_order_custom_total_weight' ) );

			// ajax handler for receive cod24 serial
			add_action( 'wp_ajax_cod24_change_status' , array( &$this , 'ajax_cod24_change_status' ) );
			add_action( 'wp_ajax_nopriv_cod24_change_status' , array( &$this , 'ajax_cod24_change_status' ) );

			// ajax handler for cod24 factor preview
			add_action( 'wp_ajax_cod24_factor_preview' , array( &$this , 'ajax_cod24_factor_preview' ) );
			add_action( 'wp_ajax_nopriv_cod24_factor_preview' , array( &$this , 'ajax_cod24_factor_preview' ) );

			// ajax handler for cod24 barcode preview
			add_action( 'wp_ajax_cod24_barcode_preview' , array( &$this , 'ajax_cod24_barcode_preview' ) );
			add_action( 'wp_ajax_nopriv_cod24_barcode_preview' , array( &$this , 'ajax_cod24_barcode_preview' ) );

			// add cod24 column to orders list admin
			add_filter( 'manage_edit-shop_order_columns' , array( &$this , 'cod24_shop_order_column' ) , 20 );
			add_action( 'manage_shop_order_posts_custom_column' , array( &$this , 'cod24_orders_list_column_content' ) , 20 , 2 );

			// Add custom cod24 order status
			add_action( 'init' , array( &$this , 'cod24_register_order_status' ) );
			add_filter( 'wc_order_statuses' , array( &$this , 'show_cod24_order_statuses_single_order_dropdown' ) );

			// Adding to admin order list bulk dropdown a custom action
			add_filter( 'bulk_actions-edit-shop_order' , array( &$this , 'send_to_cod24_bulk_action' ) , 20 , 1 );
			add_filter( 'handle_bulk_actions-edit-shop_order' , array( &$this , 'send_to_cod24_handle_bulk_action_edit_shop_order' ) , 10 , 3 );
			add_action( 'admin_notices' , array( &$this , 'send_to_cod24_bulk_action_admin_notice' ) );
			
		} // END public function __construct()

		/**
		 * load scripts for city field
		 * 
		 * @param void
		 * @return void
		 */
		public function load_scripts()
		{
			if ( is_cart() || is_checkout() || is_wc_endpoint_url( 'edit-address' ) ) 
			{
				$city_select_path = COD24_PLUGIN_URL . 'assets/js/city-select.js';
				wp_enqueue_script( 'wc-city-select', $city_select_path, array( 'jquery', 'woocommerce' ) );
		
				$states_data = COD24_API::get_states();
				$cities_data = COD24_API::get_all_cities();
				
				wp_localize_script( 'wc-city-select', 'wc_city_select_params', array(
					'states' => $states_data,
					'cities' => $cities_data,
					'ajaxurl' => admin_url('admin-ajax.php'),
					'i18n_select_city_text' => esc_attr__( 'Select an option&hellip;', 'woocommerce' )
				) );
			}
		}
		
		/**
		 * Change billing fields
		 * 
		 * @param field
		 * @param country
		 * @return fields
		 * 
		 */
		public function billing_fields( $fields, $country ) {
			$fields['billing_city']['type'] = 'city';
			return $fields;
		}
		
		/**
		 * Change shipping fields
		 * 
		 * @param field
		 * @param country
		 * @return fields
		 * 
		 */
		public function shipping_fields( $fields, $country ) {
			$fields['shipping_city']['type'] = 'city';
			return $fields;
		}
		
		/**
		 * Override for city field
		 * 
		 * @param field
		 * @param key
		 * @param args
		 * @param value
		 * 
		 * @return field
		 */
		public function form_field_city( $field, $key, $args, $value ) {
		
			// Do we need a clear div?
			if ( ( ! empty( $args['clear'] ) ) ) {
				$after = '<div class="clear"></div>';
			} else {
				$after = '';
			}
		
			// Required markup
			if ( $args['required'] ) {
				$args['class'][] = 'validate-required';
				$required = ' <abbr class="required" title="' . esc_attr__( 'required', 'woocommerce'  ) . '">*</abbr>';
			} else {
				$required = '';
			}
		
			// Custom attribute handling
			$custom_attributes = array();
		
			if ( ! empty( $args['custom_attributes'] ) && is_array( $args['custom_attributes'] ) ) {
				foreach ( $args['custom_attributes'] as $attribute => $attribute_value ) {
					$custom_attributes[] = esc_attr( $attribute ) . '="' . esc_attr( $attribute_value ) . '"';
				}
			}
		
			// Validate classes
			if ( ! empty( $args['validate'] ) ) {
				foreach( $args['validate'] as $validate ) {
					$args['class'][] = 'validate-' . $validate;
				}
			}
		
			// field p and label
			$field  = '<p class="form-row ' . esc_attr( implode( ' ', $args['class'] ) ) .'" id="' . esc_attr( $args['id'] ) . '_field">';
			if ( $args['label'] ) {
				$field .= '<label for="' . esc_attr( $args['id'] ) . '" class="' . esc_attr( implode( ' ', $args['label_class'] ) ) .'">' . $args['label']. $required . '</label>';
			}
		
			$state_key = $key == 'billing_city' ? 'billing_state' : 'shipping_state';
			$current_sc  = WC()->checkout->get_value( $state_key ) ? WC()->checkout->get_value( $state_key ) : get_user_meta( get_current_user_id(), 'billing_state', true);
		
			// Get country cities
			if( !empty( $current_sc ) )
				$cities = COD24_API::get_cities( $current_sc );
			else
				$cities = COD24_API::get_cities( 'KHZ' );
		
			if ( is_array( $cities ) ) {
		
				$field .= '<select name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" class="city_select ' . esc_attr( implode( ' ', $args['input_class'] ) ) .'" ' . implode( ' ', $custom_attributes ) . ' placeholder="' . esc_attr( $args['placeholder'] ) . '">
					<option value="">'. __( 'Select an option&hellip;', 'woocommerce' ) .'</option>';
		
				foreach ( $cities as $city_element ) {
					$field .= '<option value="' . esc_attr( $city_element['city_name'] ) . '" '.selected( $value, $city_element['city_name'], false ) . '>' . $city_element['city_name'] .'</option>';
				}
		
				$field .= '</select>';
		
			} else {
		
				$field .= '<input type="text" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) .'" value="' . esc_attr( $value ) . '"  placeholder="' . esc_attr( $args['placeholder'] ) . '" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" ' . implode( ' ', $custom_attributes ) . ' />';
			}
		
			// field description and close wrapper
			if ( $args['description'] ) {
				$field .= '<span class="description">' . esc_attr( $args['description'] ) . '</span>';
			}
		
			$field .= '</p>' . $after;
		
			return $field;
		}

		/**
		 * Overrie checkout fields
		 * 
		 * @param fields
		 * @return fields
		 */
		public function override_checkout_fields( $fields )
		{
			//$fields['billing']['billing_country']['priority'] = 40;
			$fields['billing']['billing_state']['priority'] = 50;
			$fields['billing']['billing_city']['priority'] = 60;
			$fields['billing']['billing_address_1']['priority'] = 70;
			//$fields['billing']['billing_address_2']['priority'] = 80;
			$fields['billing']['billing_postcode']['priority'] = 90;
			return $fields;
		}

		/**
		 * Override default locate fields
		 * 
		 * @param fields
		 * @return fields
		 */
		public function override_default_locale_fields( $fields )
		{
			$fields['state']['priority'] = 50;
			$fields['city']['priority'] = 60;
			$fields['address_1']['priority'] = 70;
			//$fields['address_2']['priority'] = 80;
			return $fields;
		}


		/**
		 * Woocommerce cart shipping packages
		 * 
		 * @param packages input package from another filter
		 * @return packages changed packages 
		 */
		public function cart_shipping_packages( $packages ) {

			for ( $i = 0; $i < count( $packages ); $i ++ ) {
				$packages[ $i ]['destination']['is_cod'] = WC()->session->get( 'chosen_payment_method' ) == 'cod';
				// shipping cost on delivery
				$packages[ $i ]['destination']['is_shcod'] = WC()->session->get( 'shcod_fee' ) == true;
			}
		
			return $packages;
		}

		/**
		 * Change woocommerce package rate
		 * 
		 * @param rates
		 * @return rates changed package rates
		 */
		public function hide_shipping_when_free_is_available( $rates )
		{
			$free = array();
			$chosen_payment_method = WC()->session->get('chosen_payment_method');
			$free_shipping_is_enable_in_cod = get_option('wc_cod24_free_shipping_cod_enabled');
		
			// free shipping skip all of shipping method except cod
			foreach ( $rates as $rate_id => $rate) {
				if ( 'free_shipping' === $rate->method_id) {
					if($chosen_payment_method =='cod' && $free_shipping_is_enable_in_cod!='yes')
					{
						unset( $rates[ $rate_id ] );
						continue;
					}
					else {
						$free[ $rate_id ] = $rate;
						break;
					}
				}
			}
		
			return ! empty( $free ) ? $free : $rates;
		}

		/**
		 * Save Total weight in woocommerce thankyou
		 * 
		 * @param order_id input order_id that called in woocommerce thankyou
		 * @return void
		 */
		public function checkout_save_total_weight_meta( $order_id ) {
   
			$order = new WC_Order( $order_id );
			
			if ( $order->get_total() > 100 ) {  // Define your condition here
				$total_weight = 0;
				foreach ($order->get_items() as $item) {
					if ($item['product_id'] > 0)
					{
						$quantity = $item['qty'];
						$_product = $item->get_product();
						if (!$_product->is_virtual()) {
							$weight = $_product->get_weight();
							$total_weight += $weight * $quantity;
						}
					}
				}
				update_post_meta( $order_id, '_custom_total_weight', $total_weight );
			}
		 
		}

		/**
		 * Order detail metabox
		 * 
		 * @param void
		 * @return void
		 */
		public function order_detail_metabox()
		{
			add_meta_box(
				'cod24_order_info' , 
				__('COD24 Post Gateway Info', 'cod24-shipping'),
				array( &$this , 'order_detail_callback' ),
				'shop_order',
				'side',
				'core'
			);
		}

		public function order_detail_callback()
		{
			global $post;
			$order_cod24_serial = get_post_meta($post->ID, '_cod24_serial', true) ? get_post_meta($post->ID, '_cod24_serial', true) : '';
			$order_cod24_barcode = get_post_meta($post->ID, '_cod24_barcode', true) ? get_post_meta($post->ID, '_cod24_barcode', true) : '';
			$order_cod24_status = get_post_meta($post->ID, '_cod24_last_status', true) ? get_post_meta($post->ID, '_cod24_last_status', true ) : '';
			$order_is_shcod = get_post_meta($post->ID, '_is_shipping_cost_on_delivery', true) ? get_post_meta($post->ID, '_is_shipping_cost_on_delivery', true ) : '';
			$order_custom_total_weight = get_post_meta($post->ID, '_custom_total_weight', true) ? get_post_meta($post->ID, '_custom_total_weight', true ) : '';
			//print_r(COD24_API::add_order($post->ID));
			?>
			<form id="order_barcode_form" method="post" action="">
				<?php $weight_unit = get_option('woocommerce_weight_unit'); ?>
				<div class="cod24-fields" style="margin-bottom:10px">
					<div class="label" style="margin-bottom:10px"><label for="custom_total_weight"><?php echo __('Total weight of order', 'cod24-shipping'); ?> (<?php echo $weight_unit; ?>)</label></div>
					<input type="text" class="postbox custom_total_weight" id="custom_total_weight" style="margin-bottom:0px" name="total_weight" value="<?php echo esc_attr( $order_custom_total_weight ); ?>" />
				</div>
				<div class="cod24-fields" style="margin-bottom:10px">
					<div class="label" style="margin-bottom:10px"><label for="cod24_status"><?php echo __('Change COD24 Status', 'cod24-shipping'); ?></label></div>
					<select name="cod24_status" id="cod24_status" class="postbox cod24_status" style="margin-bottom:10px">
						<option value="none"><?php _e('Select the status...','cod24-shipping'); ?></option>
						<option value="cod24-add" <?php selected( 'cod24-add', $order_cod24_status ); ?>><?php _e('Add Order', 'cod24-shipping'); ?></option>
						<option value="cod24-suspend" <?php selected( 'cod24-suspend', $order_cod24_status ); ?>><?php _e('Suspend Order', 'cod24-shipping'); ?></option>
						<option value="cod24-ready-send" <?php selected( 'cod24-ready-send', $order_cod24_status ); ?>><?php _e('Ready To Send Order', 'cod24-shipping'); ?></option>
						<option value="cod24-cancel" <?php selected( 'cod24-cancel', $order_cod24_status ); ?>><?php _e('Cancel Order', 'cod24-shipping'); ?></option>
					</select>
					<button type="button" class="button cod24_change_status" name="cod24_change_status"><?php echo __('Send To Panel', 'cod24-shipping'); ?></button>
				</div>
				<p class="serial"><?php echo __('Serial:', 'cod24-shipping'); ?> <strong><?php if(!empty($order_cod24_serial)) echo $order_cod24_serial; else echo "-"; ?></strong></p>
				<p class="barcode"><?php echo __('Post Barcode:', 'cod24-shipping'); ?> <strong><?php if(!empty($order_cod24_barcode)) echo $order_cod24_barcode; else echo "-"; ?></strong></p>
				<p class="shcod"><?php _e('Shipping cost at the destination:', 'cod24-shipping'); ?> <strong><?php if(!empty($order_is_shcod)) _e('Yes', 'cod24-shipping'); else _e('No', 'cod24-shipping'); ?></strong></p>
				<input type="hidden" class="order_barcode_nonce" name="order_barcode_nonce" value="<?php echo wp_create_nonce('order_barcode_nonce'); ?>" />
				<input type="hidden" class="order_id" name="order_id" value="<?php echo $post->ID; ?>" />
				<button type="button" class="button cod24_factorPreview" name="cod24_factorPreview" <?php if( empty($order_cod24_serial) && empty($order_cod24_barcode)) echo 'disabled="disabled"'; ?>><?php echo __('Print Factor', 'cod24-shipping'); ?></button>
			</form>
			<script>
				jQuery(document).ready(function(){
					jQuery(".cod24_change_status").on('click', function(e){
						e.preventDefault();
						var order_barcode_nonce = jQuery('.order_barcode_nonce').val();
						var order_id = jQuery('.order_id').val();
						var total_weight = jQuery('.custom_total_weight').val();
						var cod24_status = jQuery('.cod24_status').val();
						jQuery.ajax({
							type : "post",
							dataType : "json",
							url : ajaxurl,
							data : { action: "cod24_change_status", order_id : order_id, order_barcode_nonce: order_barcode_nonce, total_weight : total_weight, cod24_status : cod24_status },
							beforeSend: function() {
								if(!jQuery("#cod24_msg")[0])
									jQuery(".wp-header-end").after('<div id="cod24_msg" class="notice notice-warning"><p><?php echo __('Please wait a moment...', 'cod24-shipping') ?></p></div>');
							}, 
							success: function(response) {
								if(response.status) {
									window.location.reload();
								} else {
									jQuery("#cod24_msg").removeClass("notice-warning");
									jQuery("#cod24_msg").addClass("notice-error");
									jQuery("#cod24_msg").html(response.msg);
								}
							},
							error: function(error) {
								console.log(error);
							}
						});
					});
					jQuery(".cod24_factorPreview").on('click', function(e){
						e.preventDefault();
						var order_barcode_nonce = jQuery(".order_barcode_nonce").val();
						var order_id = jQuery(".order_id").val();
						jQuery.ajax({
							type : "post",
							dataType : "json",
							url : '<?php echo admin_url( 'admin-ajax.php' ); ?>',
							data : { action: "cod24_factor_preview", order_id : order_id, order_barcode_nonce: order_barcode_nonce },
							beforeSend: function() {
								if(!jQuery("#cod24_msg")[0])
									jQuery(".wp-header-end").after('<div id="cod24_msg" class="notice notice-warning"><p><?php echo __('Please wait a moment...', 'cod24-shipping') ?></p></div>');
							}, 
							success: function(response) {
								if(response.status) {
									jQuery("#cod24_msg").hide();
									window.open(response.url, "_blank");
								}
								else {
									jQuery("#cod24_msg").removeClass("notice-warning");
									jQuery("#cod24_msg").addClass("notice-error");
								}
								jQuery("#cod24_msg p").html(response.msg);
							},
							error: function(error) {
								console.log(error);
							}
						});
					});
					jQuery(".cod24_barcodePreview").on('click', function(e){
						e.preventDefault();
						var order_barcode_nonce = jQuery(".order_barcode_nonce").val();
						var order_id = jQuery(".order_id").val();
						jQuery.ajax({
							type : "post",
							dataType : "json",
							url : '<?php echo admin_url( 'admin-ajax.php' ); ?>',
							data : { action: "cod24_barcode_preview", order_id : order_id, order_barcode_nonce: order_barcode_nonce },
							beforeSend: function() {
								if(!jQuery("#cod24_msg")[0])
									jQuery(".wp-header-end").after('<div id="cod24_msg" class="notice notice-warning"><p><?php echo __('Please wait a moment...', 'cod24-shipping') ?></p></div>');
							}, 
							success: function(response) {
								if(response.status) {
									jQuery("#cod24_msg").hide();
									window.open(response.url, "_blank");
								}
								else {
									jQuery("#cod24_msg").removeClass("notice-warning");
									jQuery("#cod24_msg").addClass("notice-error");
								}
								jQuery("#cod24_msg p").html(response.msg);
							},
							error: function(error) {
								console.log(error);
							}
						});
					});
				});
			</script>
			<?php
		}

		/**
		 * Save custom total weight
		 */
		public function save_order_custom_total_weight( $order_id )
		{
			if ( ! isset( $_POST['total_weight'] ) ) {
				return;
			}

			$user_total_weight = floatval( sanitize_text_field( $_POST['total_weight'] ) );
		
			update_post_meta( $order_id, '_custom_total_weight', $user_total_weight );
		}
		

		/**
		 * Ajax handler for receive serial
		 * 
		 * @param void
		 * @return json $result
		 */
		public function ajax_cod24_change_status()
		{
			$result = array();
			if ( wp_verify_nonce( $_POST['order_barcode_nonce'] , 'order_barcode_nonce' ) )
			{
				$order_id = intval( $_REQUEST['order_id'] );
				$custom_total_weight = floatval( $_REQUEST['total_weight'] ); // user enter custom total weight
				$cod24_new_status = sanitize_text_field( $_REQUEST['cod24_status'] ); // user enter custom total weight

				// check and get order info
				$order = wc_get_order( $order_id );
				if( !empty( $order->get_id() ) )
				{
					// update custom total weight
					update_post_meta( $order_id, '_custom_total_weight', $custom_total_weight );

					// check cod24 status
					$cod24_serial = get_post_meta( $order_id, '_cod24_serial', true );
					$cod24_barcode = get_post_meta( $order_id, '_cod24_barcode', true );
					$cod24_last_status = get_post_meta( $order_id, '_cod24_last_status', true );

					// If the new status is 'cod24-add'
					if ( $cod24_new_status == 'cod24-add' && $cod24_last_status != 'cod24-add' )
					{
						// Check if the weight post meta is empty
						if ( empty( $custom_total_weight ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'The total weight is empty. The order status reverted to the previous status.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( !empty( $cod24_serial ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'The order has been added to the COD24 panel in the past.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$result = COD24_API::add_order( $order_id );

								if( !empty( $result['serial'] ) )
								{
									// update serial
									update_post_meta( $order_id, '_cod24_serial', $result['serial'] );

									// Add an order note for user
									$order->add_order_note(
										sprintf( __('The order successfully added to the COD24 panel. Post Serial: %s', 'cod24-shipping' ), $result['serial'] ),
										1 // Public note
									);

									$order->update_status( 'cod24-add' );

									// update cod24 last status
									update_post_meta( $order_id, '_cod24_last_status', 'cod24-add' );
								}
								else
								{
									if( !empty( $result['code'] ) && !empty( $result['message'] ) )
									{
										if( !empty( $result['errors'] ) )
										{
											$errors = '';
											foreach($result['errors'] as $error)
											{
												$errors .= $error['code'] . " - " . $error['message'];
											}

											// Add an order note for admin
											$order->add_order_note(
												sprintf( __( 'There is a problem when adding the order to the COD24 panel. Error: <br /> %s', 'cod24-shipping' ), $errors ),
												0 // Private note for admin only
											);
										}
										else
										{
											// Add an order note for admin
											$order->add_order_note(
												sprintf( __( 'There is a problem when adding the order to the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $result['code'] , $result['message'] ),
												0 // Private note for admin only
											);
										}
									}
									else
									{
										$order->add_order_note(
											sprintf( __( 'There is a problem when adding the order to the COD24 panel.', 'cod24-shipping' ) ),
											0 // Private note for admin only
										);
									}
								}
							}
						}
					}
					elseif ( $cod24_new_status == 'cod24-suspend' && $cod24_last_status != 'cod24-suspend' ) // If the new status is 'cod24-suspend'
					{
						// Check if the weight post meta is empty
						if ( empty( $cod24_last_status ) || ( !empty( $cod24_last_status ) && $cod24_last_status != 'cod24-add' ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'You should select the "Add Order" status first.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( empty( $cod24_serial ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'No Post Serial registered for this order.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$result = COD24_API::suspend_order( array( $cod24_serial ) );

								if( !empty( $result[0]['barcode'] ) && $result[0]['isSuccess'] && $result[0]['code'] == 0 )
								{
									// update barcode
									update_post_meta( $order_id, '_cod24_barcode', $result[0]['barcode'] );

									// Add an order note for user
									$order->add_order_note(
										sprintf( __('The order successfully suspended in the COD24 panel. Post Barcode: %s', 'cod24-shipping' ), $result['barcode'] ),
										1 // Public note
									);

									$order->update_status( 'cod24-suspend' );

									// update cod24 last status
									update_post_meta( $order_id, '_cod24_last_status', 'cod24-suspend' );
								}
								else
								{
									if( !empty( $result[0]['code'] ) && !empty( $result[0]['message'] ) )
									{
										// Add an order note for admin
										$order->add_order_note(
											sprintf( __( 'There is a problem when suspending the order in the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $result[0]['code'] , $result[0]['message'] ),
											0 // Private note for admin only
										);
									}
									else
									{
										$order->add_order_note(
											sprintf( __( 'There is a problem when suspending the order in the COD24 panel.', 'cod24-shipping' ) ),
											0 // Private note for admin only
										);
									}
								}
							}
						}
					}
					elseif ( $cod24_new_status == 'cod24-ready-send' && $cod24_last_status != 'cod24-ready-send' ) // If the new status is 'cod24-ready-send'
					{
						// Check if the weight post meta is empty
						if ( empty( $cod24_last_status ) || ( !empty( $cod24_last_status ) && $cod24_last_status != 'cod24-suspend' ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'You should select the "Suspend Order" status first.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( empty( $cod24_barcode ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'No Post Barcode registered for this order.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$result = COD24_API::ready_to_send_order( array( $cod24_serial ) );

								if( $result[0]['isSuccess'] && $result[0]['code'] == 0 )
								{
									// Add an order note for user
									$order->add_order_note(
										__('The order status changed to "Ready To Send" in the COD24 panel successfully.', 'cod24-shipping' ),
										1 // Public note
									);

									$order->update_status( 'cod24-ready-send' );

									// update cod24 last status
									update_post_meta( $order_id, '_cod24_last_status', 'cod24-ready-send' );
								}
								else
								{
									// Add an order note for admin
									if( !empty($result[0]['code']) && !empty( $result[0]['message'] ) )
									{
										$order->add_order_note(
											sprintf( __( 'There is a problem when changing the order status to "Ready To Send" in the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $result[0]['code'] , $result[0]['message'] ),
											0 // Private note for admin only
										);
									}
									else
									{
										$order->add_order_note(
											sprintf( __( 'There is a problem when changing the order status to "Ready To Send" in the COD24 panel.', 'cod24-shipping' ) ),
											0 // Private note for admin only
										);
									}
								}
							}
						}
					}
					elseif ( $cod24_new_status == 'cod24-cancel' && $cod24_last_status != 'cod24-cancel' ) // If the new status is 'cod24-cancel'
					{
						if( empty( $cod24_serial ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'No Post Serial registered for this order.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							$result = COD24_API::cancel_order( array( $cod24_serial ) );

							if( $result[0]['isSuccess'] && $result[0]['code'] == 0 )
							{
								// Add an order note for user
								$order->add_order_note(
									__('The order successfully cancelled in the COD24 panel.', 'cod24-shipping' ),
									1 // Public note
								);

								$order->update_status( 'cod24-cancel' );

								// update cod24 last status
								update_post_meta( $order_id, '_cod24_last_status', 'cod24-cancel' );
							}
							else
							{
								if( !empty( $result[0]['code'] ) && !empty( $result[0]['message'] ) )
								{
									// Add an order note for admin
									$order->add_order_note(
										sprintf( __( 'There is a problem when cancelling the order in the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $result[0]['code'] , $result[0]['message'] ),
										0 // Private note for admin only
									);
								}
								else
								{
									$order->add_order_note(
										sprintf( __( 'There is a problem when cancelling the order in the COD24 panel.', 'cod24-shipping' ) ),
										0 // Private note for admin only
									);
								}
							}
						}
					}

					$result = array( 'status' => true, 'url' => add_query_arg( array( 'post' => $order_id , 'action' => 'edit' ), admin_url( 'post.php' ) ) );
				}
				else
				{
					$result = array( 'status' => false , 'msg' => __('The order is invalid', 'cod24-shipping') );
				}
			}
			else
			{
				$result = array( 'status' => false , 'msg' => __('The order nonce key is invalid', 'cod24-shipping') );
			}

			echo json_encode( $result );
			die();
		}

		/**
		 * Ajax handler for cod24 get factor preview
		 * 
		 */
		public function ajax_cod24_factor_preview()
		{
			global $post;
			$current_currency = get_woocommerce_currency();
			
			if ( wp_verify_nonce( $_POST['order_barcode_nonce'] , "order_barcode_nonce" ) )
			{
				$order_id = intval( $_REQUEST['order_id'] );
				$order_info = wc_get_order( $order_id );
				if( !empty( $order_info->get_id() ) )
				{
					$cod24_serial = get_post_meta( $order_info->get_id(), '_cod24_serial', true );
					if( !empty( $cod24_serial ) )
					{
						$data[] = [ 'serial' => (int) $cod24_serial ];
						$factor_preview_result = COD24_API::factor_preview( $data );
						if( !empty( $factor_preview_result['url'] ) )
						{
							// success message
							//$message = sprintf( __('The factor is ready for printing. <a href="%s">Preview And Print Factor</a>', 'cod24-shipping'), $result['url'] );
							$result = array( 'status' => true );
						}
						else
						{
							$message = sprintf( __('There was an error in getting factor preview from COD24 panel. Error: %s ', 'cod24-shipping'), $result['code']. " - ". $result['message'] );
							$result = array('status' => false , 'msg' => $message );
						}
					}
					else
					{
						$result = array('status' => false, 'msg' => __('The Cod24 serial is invalid for this order.', 'cod24-shipping') );
					}
				}
				else
				{
					$result = array('status' => false , 'msg' => __('The order is invalid', 'cod24-shipping') );
				}
			}

			echo json_encode( $result );
			die();
		}

		/**
		 * Ajax handler for cod24 get barcode preview
		 * 
		 */
		public function ajax_cod24_barcode_preview()
		{
			global $post;
			$current_currency = get_woocommerce_currency();
			
			if ( wp_verify_nonce( $_POST['order_barcode_nonce'] , "order_barcode_nonce" ) )
			{
				$order_id = intval( $_REQUEST['order_id'] );
				$order_info = wc_get_order( $order_id );
				if( !empty( $order_info->get_id() ) )
				{
					$cod24_serial = get_post_meta( $order_info->get_id(), '_cod24_serial', true );
					if( !empty( $cod24_serial ) )
					{
						$data[] = [ 'serial' => (int) $cod24_serial ];
						$barcode_preview_result = COD24_API::barcode_preview( $data );
						if( !empty( $barcode_preview_result['url'] ) )
						{
							// success message
							$result = array( 'status' => true , 'url' => $barcode_preview_result['url'] );
						}
						else
						{
							$message = sprintf( __('There was an error in getting barcode preview from COD24 panel. Error: %s ', 'cod24-shipping'), $result['code']. " - ". $result['message'] );
							$result = array('status' => false , 'msg' => $message );
						}
					}
					else
					{
						$result = array('status' => false, 'msg' => __('The Cod24 serial is invalid for this order.', 'cod24-shipping') );
					}
				}
				else
				{
					$result = array('status' => false , 'msg' => __('The order is invalid', 'cod24-shipping') );
				}
			}

			echo json_encode( $result );
			die();
		}

		/**
		 * Add cod24 panel serial and barcode column
		 * 
		 * @param array $columns list of columns
		 * @return array $reordered_columns change and reorderd of columns
		 */
		public function cod24_shop_order_column( $columns )
		{
			$reordered_columns = array();

			// Inserting columns to a specific location
			foreach( $columns as $key => $column){
				$reordered_columns[$key] = $column;
				if( $key ==  'order_status' ){
					// Inserting after "Status" column
					$reordered_columns['cod24'] = __( 'COD24 Gateway','cod24-shipping');
				}
			}
			return $reordered_columns;
		}

		/**
		 * Add cod24 panel column content
		 * 
		 * @param string $column name of column
		 * @param int $post_id id for every row
		 * @return void
		 */
		public function cod24_orders_list_column_content( $column , $post_id )
		{
			switch ( $column )
			{
				case 'cod24' :
					// Get cod24 serial
					$cod24_serial = get_post_meta( $post_id, '_cod24_serial', true );
					printf( __("Serial: <strong>%s</strong>", 'cod24-shipping'), ( !empty( $cod24_serial ) ) ? esc_attr( $cod24_serial ) : '-' );

					// Get cod24 barcode
					$cod24_barcode = get_post_meta( $post_id, '_cod24_barcode', true );
					printf( __("<br />Post Barcode: <strong>%s</strong>", 'cod24-shipping'), ( !empty( $cod24_barcode ) ) ? esc_attr( $cod24_barcode ) : '-' );

					break;
			}
		}

		/**
		 * Cod24 register order status
		 * 
		 * @param void
		 * @return void
		 */
		public function cod24_register_order_status()
		{
			// Add addOrder to cod24 status
			register_post_status( 'wc-cod24-add', array(
				'label' => __('Added - COD24', 'cod24-shipping'),
				'public' => true,
				'exclude_from_search' => false,
				'show_in_admin_all_list' => true,
				'show_in_admin_status_list' => true,
				'label_count' => _n_noop( 'Added - COD24 <span class="count">(%s)</span>', 'Added - COD24 <span class="count">(%s)</span>', 'cod24-shipping' ),
			) );

			// Add suspendOrder to cod24 status
			register_post_status( 'wc-cod24-suspend', array(
				'label' => __('Suspended - COD24', 'cod24-shipping'),
				'public' => true,
				'exclude_from_search' => false,
				'show_in_admin_all_list' => true,
				'show_in_admin_status_list' => true,
				'label_count' => _n_noop( 'Suspended - COD24 <span class="count">(%s)</span>', 'Suspended - COD24 <span class="count">(%s)</span>', 'cod24-shipping' ),
			) );
			
			// Add ready to send post cod24 status
			register_post_status( 'wc-cod24-ready-send', array(
				'label' => __('Ready To Send - COD24','cod24-shipping'),
				'public' => true,
				'exclude_from_search' => false,
				'show_in_admin_all_list' => true,
				'show_in_admin_status_list' => true,
				'label_count' => _n_noop( 'Ready To Send - COD24 <span class="count">(%s)</span>', 'Ready To Send - COD24 <span class="count">(%s)</span>', 'cod24-shipping' ),
			) );

			// Add send order to cod24 (suspend order) status
			register_post_status( 'wc-cod24-cancel', array(
				'label' => __('Cancelled - COD24', 'cod24-shipping'),
				'public' => true,
				'exclude_from_search' => false,
				'show_in_admin_all_list' => true,
				'show_in_admin_status_list' => true,
				'label_count' => _n_noop( 'Cancelled - COD24 <span class="count">(%s)</span>', 'Cancelled - COD24 <span class="count">(%s)</span>', 'cod24-shipping' ),
			) );
		}

		/**
		 * Show cod24 order statuses in single order dropdown
		 * 
		 * @param array $order_statuses list of order statuses
		 * @param array $order_statuses changed list of order statuses
		 */
		public function show_cod24_order_statuses_single_order_dropdown( $order_statuses )
		{
			$new_order_statuses = array();

			// add new order status before processing
			foreach ($order_statuses as $key => $status) {
				$new_order_statuses[$key] = $status;
				if ('wc-completed' === $key) {
					$new_order_statuses['wc-cod24-add'] = __('Added - COD24', 'cod24-shipping');
					$new_order_statuses['wc-cod24-suspend'] = __('Suspended - COD24', 'cod24-shipping');
					$new_order_statuses['wc-cod24-ready-send'] = __('Ready To Send - COD24', 'cod24-shipping');
					$new_order_statuses['wc-cod24-cancel'] = __('Cancelled - COD24', 'cod24-shipping');
				}
			}
			return $new_order_statuses;
		}

		/**
		 * Add send to cod24 bulk action
		 * 
		 * @param array $actions list of actions
		 * @param array $actions changed action or added send_cod24 action
		 */
		public function send_to_cod24_bulk_action( $actions )
		{
			$actions['mark_cod24_add'] = __('Change status to Added - COD24', 'cod24-shipping');
			$actions['mark_cod24_suspend'] = __('Change status to Suspended - COD24', 'cod24-shipping');
			$actions['mark_cod24_ready_send'] = __('Change status to Ready To Send - COD24', 'cod24-shipping');
			$actions['mark_cod24_cancel'] = __('Change status to Cancelled - COD24', 'cod24-shipping');
			return $actions;
		}


		/**
		 * Send to cod24 handle bulk action
		 * 
		 * @param string $redirect_to url after run bulk action
		 * @param string action name of bulk action
		 * @param array $post_ids list of post ids
		 * @return run $add_query_arg redirect user to custom url
		 */
		public function send_to_cod24_handle_bulk_action_edit_shop_order( $redirect_to, $action, $post_ids )
		{
			/*if ( ! in_array( $action, 'mark_cod24_add', 'mark_cod24_suspend', 'mark_cod24_ready_send', 'mark_cod24_cancel' ) )
				return $redirect_to; // Exit
			*/
			
			$query_args = array();
			if( !empty( $post_ids ) )
			{
				
				if( $action == 'mark_cod24_add') // bulk actions for 'cod24-add'
				{
					$orders_list = array();
					foreach($post_ids as $post_id)
					{
						$custom_total_weight = get_post_meta( $post_id, '_custom_total_weight', true );
						$cod24_serial = get_post_meta( $post_id, '_cod24_serial', true );
						$cod24_last_status = get_post_meta( $post_id, '_cod24_last_status', true );

						if( !empty( $cod24_last_status ) && $cod24_last_status == 'cod24-add' )
							continue;

						$order = wc_get_order( $post_id );
						if ( empty( $custom_total_weight ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'The total weight is empty. The order status reverted to the previous status.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( !empty( $cod24_serial ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'The order has been added to the COD24 panel in the past.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$orders_list[] = $post_id;
							}
						}
					}
					
					// run add order batch
					if( !empty( $orders_list ) )
					{
						foreach( $orders_list as $order_id )
						{
							$result = COD24_API::add_order( $order_id );
							$order = wc_get_order( $order_id );

							if( !empty( $result['serial'] ) )
							{
								// update serial
								update_post_meta( $order_id, '_cod24_serial', $result['serial'] );

								// Add an order note for user
								$order->add_order_note(
									sprintf( __('The order successfully added to the COD24 panel. Post Serial: %s', 'cod24-shipping' ), $result['serial'] ),
									1 // Public note
								);

								$order->update_status( 'cod24-add' );

								// update cod24 last status
								update_post_meta( $order_id, '_cod24_last_status', 'cod24-add' );
							}
							else
							{
								if( !empty( $result['code'] ) && !empty( $result['message'] ) )
								{
									if( !empty( $result['errors'] ) )
									{
										$errors = '';
										foreach($result['errors'] as $error)
										{
											$errors .= $error['code'] . " - " . $error['message'];
										}

										// Add an order note for admin
										$order->add_order_note(
											sprintf( __( 'There is a problem when adding the order to the COD24 panel. Error: <br /> %s', 'cod24-shipping' ), $errors ),
											0 // Private note for admin only
										);
									}
									else
									{
										// Add an order note for admin
										$order->add_order_note(
											sprintf( __( 'There is a problem when adding the order to the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $result['code'] , $result['message'] ),
											0 // Private note for admin only
										);
									}
								}
								else
								{
									$order->add_order_note(
										sprintf( __( 'There is a problem when adding the order to the COD24 panel.', 'cod24-shipping' ) ),
										0 // Private note for admin only
									);
								}
							}
						}
					}
					
					$bulk_action = 'marked_cod24_add';
				}
				elseif( $action == 'mark_cod24_suspend' ) // bulk actions for 'cod24-suspend'
				{
					// get order serials
					$orders_list = array();
					foreach($post_ids as $post_id)
					{
						$cod24_serial = get_post_meta( $post_id, '_cod24_serial', true );
						$cod24_last_status = get_post_meta( $post_id, '_cod24_last_status', true );

						if( !empty( $cod24_last_status ) && $cod24_last_status == 'cod24-suspend' )
							continue;

						$order = wc_get_order( $post_id );
						// Check if the weight post meta is empty
						if ( empty( $cod24_last_status ) || ( !empty( $cod24_last_status ) && $cod24_last_status != 'cod24-add' ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'You should select the "Add Order" status first.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( empty( $cod24_serial ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'No Post Serial registered for this order.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$orders_list[] = array( 
									'order_id' => $order->get_id(),
									'serial' => $cod24_serial
								);
							}
						}
					}

					// run suspend order
					if( !empty( $orders_list ) )
					{
						$orders_serials = array_column( $orders_list, 'serial' );
						$result = COD24_API::suspend_order( $orders_serials );
	
						if( !empty( $result ) )
						{
							foreach ($result as $item)
							{
								if( !empty( $item['serial'] ) )
								{
									$founded_serial_key = array_search( $item['serial'] , array_column( $orders_list, 'serial' ) );
									$order = wc_get_order( $orders_list[$founded_serial_key]['order_id'] );
									if(!empty( $item['barcode'] ) && $item['isSuccess'] && $item['code'] == 0 )
									{
										// update barcode
										update_post_meta( $order->get_id(), '_cod24_barcode', $item['barcode'] );
		
										// Add an order note for user
										$order->add_order_note(
											sprintf( __('The order successfully suspended in the COD24 panel. Post Barcode: %s', 'cod24-shipping' ), $item['barcode'] ),
											1 // Public note
										);
		
										$order->update_status( 'cod24-suspend' );
		
										// update cod24 last status
										update_post_meta( $order->get_id(), '_cod24_last_status', 'cod24-suspend' );
									}
									else
									{
										if( !empty( $item['code'] ) && !empty( $item['message'] ) )
										{
											// Add an order note for admin
											$order->add_order_note(
												sprintf( __( 'There is a problem when suspending the order in the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $item['code'] , $item['message'] ),
												0 // Private note for admin only
											);
										}
										else
										{
											$order->add_order_note(
												sprintf( __( 'There is a problem when suspending the order in the COD24 panel.', 'cod24-shipping' ) ),
												0 // Private note for admin only
											);
										}
									}
								}
							}
						}
					}
					
					$bulk_action = 'marked_cod24_suspend';
				}
				elseif( $action == 'mark_cod24_ready_send') // bulk actions for 'cod24-ready-send'
				{
					// get order serials
					$orders_list = array();
					foreach($post_ids as $post_id)
					{
						$cod24_serial = get_post_meta( $post_id, '_cod24_serial', true );
						$cod24_barcode = get_post_meta( $post_id, '_cod24_barcode', true );
						$cod24_last_status = get_post_meta( $post_id, '_cod24_last_status', true );

						if( !empty( $cod24_last_status ) && $cod24_last_status == 'cod24-ready-send' )
							continue;

						$order = wc_get_order( $post_id );
						// Check if the weight post meta is empty
						if ( empty( $cod24_last_status ) || ( !empty( $cod24_last_status ) && $cod24_last_status != 'cod24-suspend' ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'You should select the "Suspend Order" status first.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( empty( $cod24_barcode ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'No Post Barcode registered for this order.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$orders_list[] = array( 
									'order_id' => $order->get_id(),
									'serial' => $cod24_serial
								);
							}
						}
					}

					// run ready to send order
					if( !empty( $orders_list ) )
					{
						$orders_serials = array_column( $orders_list, 'serial' );
						$result = COD24_API::ready_to_send_order( $orders_serials );
	
						if( !empty( $result ) )
						{
							foreach ($result as $item)
							{
								if( !empty( $item['serial'] ) )
								{
									$founded_serial_key = array_search( $item['serial'] , array_column( $orders_list, 'serial' ) );
									$order = wc_get_order( $orders_list[$founded_serial_key]['order_id'] );
									if( !empty( $item ) && $item['isSuccess'] && $item['code'] == 0 )
									{
										// Add an order note for user
										$order->add_order_note(
											__('The order status changed to "Ready To Send" in the COD24 panel successfully.', 'cod24-shipping' ),
											1 // Public note
										);
	
										$order->update_status( 'cod24-ready-send' );
	
										// update cod24 last status
										update_post_meta( $order->get_id(), '_cod24_last_status', 'cod24-ready-send' );
									}
									else
									{
										if( !empty( $item['code'] ) && !empty( $item['message'] ) )
										{
											// Add an order note for admin
											$order->add_order_note(
												sprintf( __( 'There is a problem when changing the order status to "Ready To Send" in the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $item['code'] , $item['message'] ),
												0 // Private note for admin only
											);
										}
										else
										{
											$order->add_order_note(
												sprintf( __( 'There is a problem when changing the order status to "Ready To Send" in the COD24 panel.', 'cod24-shipping' ) ),
												0 // Private note for admin only
											);
										}
									}
								}
							}
						}
					}

					$bulk_action = 'marked_cod24_ready_send';
				}
				elseif( $action == 'mark_cod24_cancel') // bulk actions for 'cod24-cancel'
				{
					// get order serials
					$orders_list = array();
					foreach($post_ids as $post_id)
					{
						$cod24_serial = get_post_meta( $post_id, '_cod24_serial', true );
						$cod24_last_status = get_post_meta( $post_id, '_cod24_last_status', true );

						if( !empty( $cod24_last_status ) && $cod24_last_status == 'cod24-cancel' )
							continue;

						$order = wc_get_order( $post_id );
						// Check if the weight post meta is empty
						if ( empty( $cod24_last_status ) )
						{
							// Add an order note for admin
							$order->add_order_note(
								__( 'You should select the "Add Order" status first.', 'cod24-shipping' ),
								0 // Private note for admin only
							);
						}
						else
						{
							if( empty( $cod24_serial ) )
							{
								// Add an order note for admin
								$order->add_order_note(
									__( 'No Post Serial registered for this order.', 'cod24-shipping' ),
									0 // Private note for admin only
								);
							}
							else
							{
								$orders_list[] = array( 
									'order_id' => $order->get_id(),
									'serial' => $cod24_serial
								);
							}
						}
					}

					// run cancel order
					if( !empty( $orders_list ) )
					{
						$orders_serials = array_column( $orders_list, 'serial' );
						$result = COD24_API::cancel_order( $orders_serials );
	
						if( !empty( $result ) )
						{
							foreach ($result as $item)
							{
								if( !empty( $item['serial'] ) )
								{
									$founded_serial_key = array_search( $item['serial'] , array_column( $orders_list, 'serial' ) );
									$order = wc_get_order( $orders_list[$founded_serial_key]['order_id'] );
									if( !empty( $item ) && $item['isSuccess'] && $item['code'] == 0 )
									{
										// Add an order note for user
										$order->add_order_note(
											__('The order successfully cancelled in the COD24 panel.', 'cod24-shipping' ),
											1 // Public note
										);
	
										$order->update_status( 'cod24-cancel' );
	
										// update cod24 last status
										update_post_meta( $order->get_id(), '_cod24_last_status', 'cod24-cancel' );
									}
									else
									{
										if( !empty( $item['code'] ) && !empty( $item['message'] ) )
										{
											// Add an order note for admin
											$order->add_order_note(
												sprintf( __( 'There is a problem when cancelling the order in the COD24 panel. Error: %s - %s', 'cod24-shipping' ), $item['code'] , $item['message'] ),
												0 // Private note for admin only
											);
										}
										else
										{
											$order->add_order_note(
												sprintf( __( 'There is a problem when cancelling the order in the COD24 panel.', 'cod24-shipping' ) ),
												0 // Private note for admin only
											);
										}
									}
								}
							}
						}
					}
					
					$bulk_action = 'marked_cod24_cancel';
				}
			}

			return $redirect_to = add_query_arg( array(
				'bulk_action' => $bulk_action,
				'processed_count' => count( $post_ids ),
				'processed_ids' => implode( ',', $post_ids ),
			), $redirect_to );
		}

		/**
		 * Send to cod24 bulk action admin notice
		 * 
		 * @param void
		 * @return void
		 */
		public function send_to_cod24_bulk_action_admin_notice()
		{
			if ( empty( $_REQUEST['bulk_action'] ) ) return; // Exit
		
			$count = intval( $_REQUEST['processed_count'] );

			if( !empty( $_REQUEST['bulk_action'] ) )
			{
				if( $_REQUEST['bulk_action'] == 'marked_cod24_add' )
				{
					printf( '<div id="message" class="updated fade"><p>' .
						_n( '%s Order processed for changing status to "Added - COD24".',
						'%s Orders processed for changing status to "Added - COD24".',
						$count,
						'cod24-shipping'
					) . '</p></div>', $count );
				}
				elseif( $_REQUEST['bulk_action'] == 'marked_cod24_suspend' )
				{
					printf( '<div id="message" class="updated fade"><p>' .
						_n( '%s Order processed for changing status to "Suspended - COD24".',
						'%s Orders processed for changing status to "Suspended - COD24".',
						$count,
						'cod24-shipping'
					) . '</p></div>', $count );
				}
				elseif( $_REQUEST['bulk_action'] == 'marked_cod24_ready_send' )
				{
					printf( '<div id="message" class="updated fade"><p>' .
						_n( '%s Order processed for changing status to "Ready To Send - COD24".',
						'%s Orders processed for changing status to "Ready To Send - COD24".',
						$count,
						'cod24-shipping'
					) . '</p></div>', $count );
				}
				elseif( $_REQUEST['bulk_action'] == 'marked_cod24_cancel' )
				{
					printf( '<div id="message" class="updated fade"><p>' .
						_n( '%s Order processed for changing status to "Cancelled - COD24".',
						'%s Orders processed for changing status to "Cancelled - COD24".',
						$count,
						'cod24-shipping'
					) . '</p></div>', $count );
				}
			}
		}

	} // END class COD24_Shipping
} // END if(!class_exists('COD24_Shipping'))

if (class_exists('COD24_WOO')) {
	// instantiate the plugin class
	new COD24_WOO();
}